/**
 * Copyright 2014 (c) Baidu, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 */
#ifndef BAIDU_BOS_CPPSDK_TAG_RESPONSE_LIST_TAG_RESPONSE_H
#define BAIDU_BOS_CPPSDK_TAG_RESPONSE_LIST_TAG_RESPONSE_H

#include <vector>
#include <string>
#include <sstream>

#include "bcesdk/model/json_response.h"
#include "bcesdk/util/util.h"
#include <json/json.h>

BEGIN_CPPSDK_NAMESPACE

struct TagResult {
    int64_t tag_id;
    std::string tag_key;
    std::string tag_value;
    std::string account_id;
    std::string bucket;
    std::string region;
    std::string action;
    std::string action_time;
};

class ListTagResponse : public JsonResponse {
public:
    ListTagResponse() {
    }

    virtual ~ListTagResponse() {}

    const int &total_count() const {
        return _total_count;
    }

    const int &page_num() const {
        return _page_num;
    }
    const int &page_size() const {
        return _page_size;
    }

    const std::string &timestamp() const {
        return _timestamp;
    }

    std::vector<TagResult> &tag_results() {
        return _tag_results;
    }

    virtual int deserialize(Json::Value &root) {
        _total_count = root["totalCount"].asInt();
        _service = root["serviceType"].asString();
        _page_num = root["pageNo"].asInt();
        _page_size = root["pageSize"].asInt();
        _timestamp = root["timestamp"].asString();

        Json::Value &results = root["result"];
        _tag_results.resize(results.size());
        for (unsigned int i = 0; i < results.size(); i++){
            TagResult& t = _tag_results[i];

            t.tag_id = results[i]["tagId"].asInt64();
            t.tag_key = results[i]["tagKey"].asString();
            t.tag_value = results[i]["tagValue"].asString();
            t.account_id = results[i]["accountId"].asString();
            t.bucket = results[i]["resourceId"].asString();
            t.region = results[i]["region"].asString();
            t.action = results[i]["action"].asString();
            t.action_time = results[i]["actionTime"].asString();
        }

        return RET_OK;
    }


private:
    int _total_count;
    std::string _service;
    int _page_num;
    int _page_size;
    std::string _timestamp;
    std::vector<TagResult> _tag_results;
};

END_CPPSDK_NAMESPACE
#endif
/* vim: set expandtab ts=4 sw=4 sts=4 tw=100: */
