/**
 * bosfs - A fuse-based file system implemented on Baidu Object Storage(BOS)
 *
 * Copyright (c) 2016 Baidu.com, Inc. All rights reserved.
 *
 * @file    common.h
 * @author  Song Shuangyang(songshuangyang@baidu.com)
 * @date    2016.8
 * @brief   Declare the common variables, macros and so on.
 **/
#ifndef BAIDU_BOS_BOSFS_COMMON_H
#define BAIDU_BOS_BOSFS_COMMON_H

#include <stdint.h>
#include <sys/types.h>

#include <string>
#include <map>
#include <atomic>

#ifndef VERSION
#include "config.h"
#endif

#define BEGIN_FS_NAMESPACE \
namespace baidu { \
namespace bos { \
namespace bosfs {

#define END_FS_NAMESPACE }}}
#define bosfs_ns baidu::bos::bosfs

#define DEFAULT_ENDPOINT "bj.bcebos.com"

BEGIN_FS_NAMESPACE

enum EMetaStrategy {
    META_LOCAL           = 1,
    META_REMOTE          = 2,
    META_MERGE           = 3,
};

enum ERwStrategy {
    S_DEFAUL             = 0,
    S_MEM_SEQUENCE_RW    = 1,   //sequence read/write(download/upload)
    S_RANDOM_RW          = 2,   //random read & write when buffer cover all file size
    S_PRELOAD_BEFORE_W   = 3,   //load all file from bos before write
};

enum EMemCacheOp {
    OP_DEFAULT             = 0,
    OP_READ                = 1,   //read
    OP_WRITE               = 2,   //write
};

// Common global variable
extern std::string        g_program_name;
extern std::string        g_endpoint;
extern std::string        g_bucket;
extern std::string        g_bucket_prefix;
extern int                g_bucket_type;
extern std::string        g_ak;
extern std::string        g_sk;
extern std::string        g_sts_token;
extern std::string        g_storage_class;
extern std::string        g_credentials_path;
extern EMetaStrategy      g_meta_strategy;
extern const int          g_meta_strategy_local;
extern const int          g_meta_strategy_remote;

extern const int          g_meta_strategy_merge;
extern int64_t            g_multipart_size;
extern int                g_multipart_parallel;
extern int64_t            g_multipart_threshold;
extern int64_t            g_multipart_min_threshold;
extern int64_t            g_block_size;
extern int64_t            g_preload_blocks;
extern int64_t            g_connect_timeout; // ms
extern int64_t            g_client_timeout; // ms

// Variables for command arguments
extern time_t             g_mount_time;
extern uid_t              g_mount_uid;
extern gid_t              g_mount_gid;
extern mode_t             g_mount_mode;
extern mode_t             g_mount_umask;
extern bool               g_is_mount_umask;
extern std::string        g_mountpoint;
extern bool               g_allow_other;
extern uid_t              g_bosfs_uid;
extern gid_t              g_bosfs_gid;
extern mode_t             g_bosfs_mask;
extern bool               g_is_bosfs_uid;
extern bool               g_is_bosfs_gid;
extern bool               g_is_bosfs_umask;
extern bool               g_remove_cache;
extern bool               g_create_bucket;
extern bool               g_is_direct_io;

extern int64_t            g_mem_cache_per_size;
extern int64_t            g_mem_cache_size_limit;
extern std::atomic<int64_t> g_mem_cache_size_use;
extern ERwStrategy        g_rw_strategy;
extern int64_t            g_meta_expire_delay_us;

END_FS_NAMESPACE

#endif
