/**
 * bosfs - A fuse-based file system implemented on Baidu Object Storage(BOS)
 * MemBuf: memory
 *
 * Copyright (c) 2021 Baidu.com, Inc. All rights reserved.
 *
 * @file    mem_buf.h
 * @author  caixiayu@baidu.com
 * @date    2021.07
 **/

#ifndef BAIDU_BOS_BOSFS_M_BUF_H
#define BAIDU_BOS_BOSFS_M_BUF__H

#include "common.h"
#include "util.h"

BEGIN_FS_NAMESPACE

/**
 * Manage the object memory buffer when use mem cache model.
**/
class MemBuf {
public:
    MemBuf();
    ~MemBuf();

    void init(int64_t buf_size, int64_t file_size, int64_t pos = 0);
    int64_t read(char *buffer, int64_t size, int64_t offset);
    int64_t write(const char *buffer, int64_t size, int64_t offset);
    int adjust_for_load(uint64_t need_load_size, int64_t offset);

    int64_t file_size() const {
        return _file_size;
    }
    int set_file_size(int64_t size) {
        _file_size = size;
    }

    int64_t size() const {
        return _size;
    }

    // never enter
    int64_t seek(int64_t pos) {
        if (pos < 0 || pos > _size) {
            return -1;
        }
        _pos = pos;
        return pos;
    }

    int64_t tell() const {
        return _pos;
    }

    int64_t data_pos() const {
        return _data_pos;
    }

    int64_t remain_size(int64_t size) const {
        int64_t cur_remain_size = _size - _pos;
        if (size > cur_remain_size) {
            size = cur_remain_size;
        }
        return size;
    }

    int64_t remain_load_size() const {
        return _size - _data_pos;
    }

    char *buffer() {
        return _buffer;
    }

    ERwStrategy rw_strategy() const {
        return _rw_strategy;
    }

    void set_rw_strategy(ERwStrategy strategy) {
        _rw_strategy = strategy;
    }

    bool check_sequence(EMemCacheOp cur_op, int64_t cur_offset) {
        return (_last_op_pos + _last_op_size == cur_offset)
        && (_last_op != EMemCacheOp::OP_DEFAULT ? _last_op == cur_op : true);
    }

    void update_last_op(EMemCacheOp op, int64_t offset, int64_t size) {
        _last_op = op;
        _last_op_pos = offset;
        _last_op_size = size;
    }


private:
    char*          _buffer;
    int64_t        _size;
    int64_t        _pos;
    int64_t        _file_size;      //cur file size

    //for load data form bos
    int64_t        _data_pos;       //cur real data pos
    int64_t        _pre_load_pos;   //next load offset

    //for check sequence in S_MEM_SEQUENCE_RW
    EMemCacheOp    _last_op;
    int64_t        _last_op_size;
    int64_t        _last_op_pos;

    ERwStrategy    _rw_strategy;
};

END_FS_NAMESPACE

#endif
/* vim: set expandtab ts=4 sw=4 sts=4 tw=100: */