/**
 * bosfs - A fuse-based file system implemented on Baidu Object Storage(BOS)
 *
 * Copyright (c) 2016 Baidu.com, Inc. All rights reserved.
 *
 * @file    object_page.h
 * @author  Song Shuangyang(songshuangyang@baidu.com)
 * @date    2016.9
 **/

#ifndef BAIDU_BOS_BOSFS_OBJECT_PAGE_H
#define BAIDU_BOS_BOSFS_OBJECT_PAGE_H

#include <list>

#include "common.h"
#include "util.h"

BEGIN_FS_NAMESPACE

class DataCacheEntity;
class StatCacheFile;

/**
 * Object block page information
 */
class ObjectPage {
public:
    ObjectPage(off_t start = 0, size_t size = 0, bool is_loaded = false) :
            _offset(start), _bytes(size), _loaded(is_loaded) {
        // nothing to do
    }

    off_t next() const {
        return _offset + _bytes;
    }

    off_t end() const {
        return 0 < _bytes ? _offset + _bytes - 1 : 0;
    }

    off_t get_offset() const {
        return _offset;
    }

    void set_offset(off_t s) {
        _offset = s;
    }

    off_t get_bytes() const {
        return _bytes;
    }

    void set_bytes(off_t s) {
        _bytes = s;
    }

    bool get_loaded() const {
        return _loaded;
    }

    void set_loaded(bool s) {
        _loaded = s;
    }

private:
    off_t _offset;
    size_t _bytes;
    bool _loaded;
};

/**
 * Manage the object for loading, modifying and area
 */
class ObjectPageList {
public:
    friend class DataCacheEntity;
    typedef std::list<ObjectPage *> self_type;

    static void free_list(self_type &list);

    explicit ObjectPageList(size_t size = 0, bool loaded = false);
    ~ObjectPageList();
    bool init(size_t size, bool loaded);
    size_t get_size() const;
    bool resize(size_t size, bool loaded);
    bool is_page_loaded(off_t start = 0, size_t size = 0) const;
    bool set_page_loaded_status(off_t start, size_t size, bool loaded = true, bool compress = true);
    bool find_unloaded_pate(off_t start, off_t &ret_start, size_t &ret_size) const;
    size_t get_total_unloaded_page_size(off_t start = 0, size_t size = 0) const;
    int get_unloaded_pages(self_type &unloaded_list, off_t start = 0, size_t size = 0) const;
    bool serialize(StatCacheFile &file, bool is_output);
    void dump();

private:
    void clear();
    bool compress();
    bool parse(off_t new_pos);

private:
    self_type _pages;
};



END_FS_NAMESPACE

#endif
/* vim: set expandtab ts=4 sw=4 sts=4 tw=100: */